get_password() {
    local pgpw_1
    local pgpw_2
    printf "Enter a password to be used for the Agent database: "
    stty -echo
    read pgpw_1
    printf "\n"
    if [[ -z $pgpw_1 ]]; then
        printf "Password string may not be empty. Please try again.\n"
        return
    fi
    if [[ ! $pgpw_1 =~ .*[a-z].* ]] || [[ ! $pgpw_1 =~ .*[A-Z].* ]] || [[ ! $pgpw_1 =~ .*[0-9].* ]] || [[ ! ${#pgpw_1} -ge 10 ]] || [[ ! ${#pgpw_1} -le 64 ]]; then
        printf "Passwords must contain one lowercase letter, one uppercase letter, and one numerical digit, and must be between 10 and 64 characters long. Please ensure your password meets these requirements and try again.\n"
        return
    fi
    printf "Re-enter chosen password: "
    read pgpw_2
    stty echo
    printf "\n"
    if [[ $pgpw_1 != $pgpw_2 ]]; then
        printf "Password strings must match. Please try again.\n"
        return
    else
        pgpw=$pgpw_1
        return
    fi
}

create_docker_compose_file() {
    local dc_content="@dockercompose@"
    printf "$dc_content" > "$composefile"
}

populate_extra_hosts() {
    sed -i.bak "65i\\
    extra_hosts:
" "$composefile"
    sed -i.bak "66i\\
      - \"$xf_portal_hostname:$xf_portal_ip\"
" $composefile
    sed -i.bak "67i\\
      - \"$xf_platform_hostname:$xf_platform_ip\"
" $composefile
    sed -i.bak "68i\\
      - \"$repo_hostname:$repo_ip\"
" $composefile
    sed -i.bak "69i\\
      - \"$rmq_hostname:$rmq_ip\"
" $composefile
  rm -f "$composefile.bak"
}

main() {
    local xf_portal_ip=""
    local xf_portal_hostname="xf.test.fortisoar.forticloud.com"
    local xf_platform_ip=""
    local xf_platform_hostname=""
    local rmq_ip=""
    local rmq_hostname=""
    local repo_ip="10.132.255.153"
    local repo_hostname="repo.fortisoar.fortinet.com"
    if [[ -z $xf_portal_ip ]] || [[ -z $xf_platform_ip ]] || [[ -z $xf_platform_hostname ]] || [[ -z $rmq_ip ]] || [[ -z $rmq_hostname ]]; then
        printf "One or more extra_hosts variables is not set. Please check these and try again. Current values are:\n"
        printf "xf_portal_ip: $xf_portal_ip\n"
        printf "xf_portal_hostname: $xf_portal_hostname\n"
        printf "xf_platform_ip: $xf_platform_ip\n"
        printf "xf_platform_hostname: $xf_platform_hostname\n"
        printf "rmq_ip: $rmq_ip\n"
        printf "rmq_hostname: $rmq_hostname\n"
        printf "repo_ip: $repo_ip\n"
        printf "repo_hostname: $repo_hostname\n"
        exit 1
    fi
    local pgpw=""
    while [[ -z $pgpw ]]
    do
        get_password
    done
    local composefile="docker-compose-@agentName@.yaml"
    create_docker_compose_file
    populate_extra_hosts

    dbpass=$pgpw docker compose --project-name agent-@agentName@ -f $composefile up --detach
}

main